#include "picoled/buffer.h"

#include <pico/stdio.h>

namespace
{

constexpr int FRACTIONAL_BITS = 16;

}

namespace picoled
{

fixed::fixed() :
  value(0)
{}

fixed::fixed(float value) :
  value(value * (1 << FRACTIONAL_BITS))
{}

fixed::operator int16_t() const
{
  return value >> FRACTIONAL_BITS;
}

fixed fixed::operator-() const
{
  fixed f;
  f.value = -value;
  return f;
}

fixed fixed::operator~() const
{
  fixed f;
  f.value = ~value;
  return f;
}

fixed fixed::operator+(const fixed& other) const
{
  fixed copy = *this;
  copy += other;
  return copy;
}

fixed fixed::operator-(const fixed& other) const
{
  fixed copy = *this;
  copy -= other;
  return copy;
}

fixed fixed::operator*(const fixed& other) const
{
  fixed copy = *this;
  copy *= other;
  return copy;
}

fixed fixed::operator/(const fixed& other) const
{
  fixed copy = *this;
  copy /= other;
  return copy;
}

fixed& fixed::operator+=(const fixed& other)
{
  value += other.value;
  return *this;
}

fixed& fixed::operator-=(const fixed& other)
{
  value -= other.value;
  return *this;
}

fixed& fixed::operator*=(const fixed& other)
{
  value = (int64_t(value) * int64_t(other.value)) >> FRACTIONAL_BITS;
  return *this;
}

fixed& fixed::operator/=(const fixed& other)
{
  value = (int64_t(value) << FRACTIONAL_BITS) / other.value;
  return *this;
}

bool fixed::operator>(const fixed& other) const { return value > other.value; }
bool fixed::operator>=(const fixed& other) const { return value >= other.value; }
bool fixed::operator<(const fixed& other) const { return value < other.value; }
bool fixed::operator<=(const fixed& other) const { return value <= other.value; }
bool fixed::operator==(const fixed& other) const { return value == other.value; }

fixed fixed::min(fixed a, fixed b)
{
  return a < b ? a : b;
}

fixed fixed::max(fixed a, fixed b)
{
  return a > b ? a : b;
}

}
